//	gmap.js (+ gmap.html + gmap/search.gif + gmap/gguy.gif)
//	Copyright 2009 CMONOS Co.Ltd.
//	Version: 1.3.0
//	License: MIT License 
//	※ Google Maps API および各種マーカー画像に関する全ての権利は米 Google, Inc が保有しています。
//
//	URI: http://cmonos.jp/
//	Last Modified: 2010-08-30
//	Author: CMONOS Co.Ltd.
//
//	【ストリートビュー関連コードのベースとしたデモ】
//	Street View Data Querying (Example by Tom Manshreck)
//	http://gmaps-samples.googlecode.com/svn/trunk/streetview/streetview_layer.html
//
//	【使い方】
//	次のような <iframe> タグで、地図を埋め込みます。
//
//	<iframe class="gmap-frame" id="gmap-canvas" src="./gmap.html?key=API_KEY&om=1&z=4&c=36.315125,137.504883"  width="500" height="400" scrolling="no" marginwidth="0" marginheight="0" frameborder="0">
//	<a href="./gmap.html?key=API_KEY&om=1&z=4&c=36.315125,137.504883" target="_blank">新しいウィンドウで地図を開く</a>
//	</iframe>
//
//	【地図呼び出しオプション】
//	key		GoogleMaps API キー(http://code.google.com/apis/maps/signup.html で取得します)
//	om		1=概観地図(右下に表示される小さな四角い概観地図)を表示する場合
//	z		拡大倍率(0-17)
//	t		地図のタイプ(k=衛星写真/h=地図と衛星の合成/設定なし=通常の地図)
//	c		中心座標の緯度,中心座標の経度
//	m		中心座標コメント
//	mw		中心座標コメントを表示する情報ウィンドウの幅
//	mk		カスタムマーカー画像URL
//	mks		カスタムマーカー影画像URL
//	mka		カスタムマーカーが地図に固定される水平位置（画像に対して left center right で指定します）
//	mkv		カスタムマーカーが地図に固定される垂直位置（画像に対して top center bottom で指定します）
//	s		1=地図検索欄を表示する
//	sv		1=ストリートビューを表示する
//	url		kml ファイルの URL
//	fit		1=kml ファイルに合わせて中心座標と拡大倍率を調整する
//	d		kml ファイルに記述された線分の距離を表示するかどうか
//			(1=始点から終点までの総延長を表示する/2=総延長に加えて始点と終点の直線距離を表示する)
//	width	地図の幅(ピクセル)
//	height	地図の高さ(ピクセル)
//	tr		情報ウィンドウ内リンクのターゲット p=_parent/s=_self/b=_blank/t=_top
//	i		1=index 用KMLを表示する(リンクつきのPlacemarkerのみで構成されたKML)
//	ns		1=index 用KMLを表示する際、マーカーを親HTMLのリンクと連動させない
//	opn		地図を埋め込んだ記事の URL/URLが一致したマーカーの情報ウィンドウを開きます
//	r		1=地図を移動または拡大縮小する度に kml を再読み込みする(必ず fit を無効化します)
//	sf		kml 再検索フォームのID
//	lb		表示している Placemarker の一覧を表示するリスト(<ul> or <ol>)を含むブロック(リストが空の時非表示にします)
//	li		表示している Placemarker の一覧を表示するリスト(<ul> or <ol>)
//
//	【以下は呼びだし元ページのフォームに地図の情報を書き込む場合に使用するオプションです】
//	z_field		表示中の地図の拡大倍率を収める入力欄のID
//	t_field		表示中の地図の地図のタイプを収める入力欄のID
//	c_field		表示中の地図の中心座標を収める入力欄のID
//	m_field		表示中の地図の中心座標コメントが収められている入力欄のID
//	mk_field	カスタムマーカー画像URLが収められている入力欄のID
//	mk_disabled	カスタムマーカー画像URLが収められている入力欄を有効化するチェックボックスのID(オプション)
//	fit_field	表示中の地図を KML ファイルにあわせて調整するかどうかの設定が収められているチェックボックスのID
//	addr_field	地図検索欄に読み込む住所が収められている住所入力欄のIDのカンマ区切りリスト(都道府県,市区群,番地,ビル)

//	外部ドメインからのアクセスを許可する場合は下記を false にします。
var domain_check = true;

//	デフォルトのマーカーアイコン
var icon_cache = new Array();
readIcon("http://labs.google.com/ridefinder/images/mm_20_red.png");
readIcon("http://labs.google.com/ridefinder/images/mm_20_shadow.png");
function getGIcon() {
	var g_icon = new GIcon();
	g_icon.image = "http://labs.google.com/ridefinder/images/mm_20_red.png";
	g_icon.shadow = "http://labs.google.com/ridefinder/images/mm_20_shadow.png";
	g_icon.iconSize = new GSize(12, 20);
	g_icon.shadowSize = new GSize(22, 20);
	g_icon.iconAnchor = new GPoint(6, 20);
	g_icon.infoWindowAnchor = new GPoint(5, 1);
	miniicon_disabled = false;
	return g_icon;
}

//	定規アイコン
function getMeasureIcon() {
	var m_icon = new GIcon();
	m_icon.image = "http://maps.gstatic.com/mapfiles/kml/pal5/icon5.png";
	m_icon.shadow = "http://maps.gstatic.com/mapfiles/kml/pal5/icon5s.png";
	m_icon.iconSize = new GSize(32,32);
	m_icon.shadowSize = new GSize(59,32);
	m_icon.iconAnchor = new GPoint(16,16);
	m_icon.infoWindowAnchor = new GPoint(16,1);
	return m_icon;
}

//	選択マーカー画像
readIcon("http://labs.google.com/ridefinder/images/mm_20_orange.png");
function toggleGIcon(this_marker,is_selected) {
	this_marker.setImage((is_selected) ? "http://labs.google.com/ridefinder/images/mm_20_orange.png" : "http://labs.google.com/ridefinder/images/mm_20_red.png");
}

//	ストリートビューマン
function getGuyIcon() {
	var guy_icon = new GIcon(G_DEFAULT_ICON);
	guy_icon.image = "http://maps.gstatic.com/mapfiles/cb/man_fly_right.png";
	guy_icon.transparent = "http://maps.gstatic.com/mapfiles/cb/man-pick.png";
	guy_icon.shadow = null;
	guy_icon.imageMap = [
		26,13, 30,14, 32,28, 27,28, 28,36, 18,35, 18,27, 16,26,
		16,20, 16,14, 19,13, 22,8
	];
	guy_icon.iconSize = new GSize(49, 52);
	guy_icon.iconAnchor = new GPoint(25, 35);
	guy_icon.infoWindowAnchor = new GPoint(25, 5);
	return guy_icon;
}

var q = urlDecode(self.document.location.search);
var api_enabled = false;
var map;
var map_center;
var center_marker = null;
var geocoder_client = null;
var measure_icon = null;
var default_icon = null;
var streetview_client = null;
var streetview_overlay = null;
var streetview_man = null;
var streetview_man_location = null;
var streetview_man_last = null;
var streetview_man_pov = {};
var kml_overlay = null;
var kml_objects = null;
var _caller = (parent && parent != self) ? parent : (opener) ? opener : self; 
if (q["key"] != null && q["key"] != "") {
	if (!domain_check || _caller.document.domain == self.document.domain) api_enabled = true;
	if ((navigator.userAgent.match(/MSIE\s*(\d+\.\d+)/) && parseFloat(RegExp.$1) < 5.5) || (navigator.userAgent.match(/Safari\/(\d+)/) && parseFloat(RegExp.$1) < 400)) api_enabled = false;
	if (api_enabled) self.document.write('<script src="http://maps.google.com/maps?file=api&amp;v=2.x&amp;key=' + q["key"] + '" type="text/javascript"></script>');
}
var z_field = null;
var t_field = null;
var c_field = null;
var m_field = null;
var mk_field = null;
var mkc_field = null;
var mka_field = null;
var mkv_field = null;
var fit_field = null;
var addr_field_ids = null;
var mw = (q["mw"] != null && q["mw"] > 100) ? q["mw"] : 280;
if (q["z_field"] != null && q["z_field"] != "" && _caller.document.getElementById(q["z_field"])) z_field = _caller.document.getElementById(q["z_field"]);
if (q["t_field"] != null && q["t_field"] != "" && _caller.document.getElementById(q["t_field"])) t_field = _caller.document.getElementById(q["t_field"]);
if (q["c_field"] != null && q["c_field"] != "" && _caller.document.getElementById(q["c_field"])) c_field = _caller.document.getElementById(q["c_field"]);
if (q["m_field"] != null && q["m_field"] != "" && _caller.document.getElementById(q["m_field"])) m_field = _caller.document.getElementById(q["m_field"]);
if (q["mk_field"] != null && q["mk_field"] != "" && _caller.document.getElementById(q["mk_field"])) mk_field = _caller.document.getElementById(q["mk_field"]);
if (q["mkc_field"] != null && q["mkc_field"] != "" && _caller.document.getElementById(q["mkc_field"])) mkc_field = _caller.document.getElementById(q["mkc_field"]);
if (q["mka_field"] != null && q["mka_field"] != "" && _caller.document.getElementById(q["mka_field"])) mka_field = _caller.document.getElementById(q["mka_field"]);
if (q["mkv_field"] != null && q["mkv_field"] != "" && _caller.document.getElementById(q["mkv_field"])) mkv_field = _caller.document.getElementById(q["mkv_field"]);
if (q["fit_field"] != null && q["fit_field"] != "" && _caller.document.getElementById(q["fit_field"])) fit_field = _caller.document.getElementById(q["fit_field"]);
if (q["addr_field"] != null && q["addr_field"] != "") addr_field_ids = q["addr_field"].split(/,/);
var field_enabled = (z_field != null || t_field != null || c_field != null || m_field != null || fit_field != null) ? true : false;
var kml_url = null;
var base_url = null;
var search_form = null;
var list_box = null;
var list = null;
var miniicon_disabled = false;
var kml_by_myself = (q["i"] != null && parseInt(q["i"]) > 0) ? true : false;
var nosync = (q["ns"] != null && parseInt(q["ns"]) > 0) ? true : false;
if (q["lb"] != null && q["lb"] != "" && _caller.document.getElementById(q["lb"])) list_box = _caller.document.getElementById(q["lb"]);
if (q["li"] != null && q["li"] != "" && _caller.document.getElementById(q["li"])) list = _caller.document.getElementById(q["li"]);
if (q["r"] != null && q["r"] == 1) q["fit"] = null;
if (q["mk"] != null) readIcon(q["mk"]);
if (q["mks"] != null) readIcon(q["mks"]);
if (mk_field != null) {
	if (mk_field.type.search(/select/) != -1) {
		for (var i=0; i < mk_field.options.length; i++) {
			if (mk_field.options[i].value != null && mk_field.options[i].value != "") {
				readIcon(mk_field.options[i].value);
			}
		}
	} else if (mk_field.value != null && mk_field.value != "") {
		readIcon(mk_field.value);
	}
};
function readIcon(this_url) {
	var i = icon_cache.length;
	icon_cache[i] = new Image();
	icon_cache[i].src = this_url;
}
function load() {
	if (api_enabled && GBrowserIsCompatible()) {
		var z = 4;
		var center = new Array(36.315125,137.504883);
		var w;
		var h;
		if (q["width"] != null && q["width"] != "" && !isNaN(q["width"]) && q["height"] != null && q["height"] != "" && !isNaN(q["height"])) {
			w = parseInt(q["width"]);
			h = parseInt(q["height"]);
		} else {
			w = ((self.innerWidth) ? self.innerWidth : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientWidth : self.document.body.clientWidth));
			h = ((self.innerHeight) ? self.innerHeight : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientHeight : self.document.body.clientHeight))
			self.onresize = function () { fitWindow(); };
		}
		map = new GMap2(self.document.getElementById("map"),{ size : new GSize(w, h) });
		map.enableScrollWheelZoom();  
		map.enableContinuousZoom();
		if (q["c"] != null && q["c"] != "") center = q["c"].split(/,/);
		if (q["z"] != null && q["z"] != "" && !isNaN(q["z"])) z = parseInt(q["z"]);
		map_center = new GLatLng(center[0],center[1]);
		map.setCenter(map_center, z);
		if (h > 320) {
			map.addControl( new GLargeMapControl() );
		} else {
			map.addControl( new GSmallMapControl() );
		}
		if (w > 320) {
			map.addControl( new GMapTypeControl() );
		} else {
			map.addControl( new GMenuMapTypeControl(true) );
		}
		map.addControl(new GScaleControl());
		if (q["om"] != null && q["om"] == 1) map.addControl(new GOverviewMapControl());
		if (q["t"] != null && q["t"] != "") map.setMapType((q["t"] == "k") ? G_SATELLITE_MAP : (q["t"] == "h") ? G_HYBRID_MAP : G_NORMAL_MAP);
		if ((m_field != null) || (q["m"] != null && q["m"] != "")) {
			if (m_field != null) {
				makeCenterMarker(true);
				if (mk_field != null) mk_field.onchange = function () { makeCenterMarker(true); };
				if (mkc_field != null) mkc_field.onclick = function () { makeCenterMarker(true); };
				if (mka_field != null) mka_field.onchange = function () { makeCenterMarker(true); };
				if (mkv_field != null) mkv_field.onchange = function () { makeCenterMarker(true); };
			} else {
				center_marker = new GMarker(map_center,{icon : ((q["mk"] != null && q["mk"] != "") ? getCustomGIcon(q["mk"],q["mka"],q["mkv"],q["mks"]) : getGIcon())});
				GEvent.addListener(
					center_marker,
					"click",
					function () {
						center_marker.openInfoWindowHtml('<div style="font-size: 14px">' + q["m"] + '</div>',{ maxWidth : mw })
					}
				);
				map.addOverlay(center_marker);
			}
		}
		if (q["url"] != null && q["url"] != "") {
			kml_url = q["url"];
			base_url = q["url"].replace(/\?.*$/,'');
		} else if (q["sf"] != null && q["sf"] != "" && _caller.document.getElementById(q["sf"])) {
			search_form = _caller.document.getElementById(q["sf"]);
			base_url = search_form.action;
			kml_url = base_url + '?' + urlEncode(getHashFromForm(search_form));
		}
		if (kml_url != null) {
			refreshKml();
			if (search_form != null) {
				search_form.onsubmit = function () {
					kml_url = base_url + '?' + urlEncode(getHashFromForm(this));
					refreshKml();
					return false;
				};
			}
			if (q["r"] != null && q["r"] == 1) {
				GEvent.addListener(
					map,
					'moveend',
					function () {
						if (map.getInfoWindow().isHidden()) {
							refreshKml();
						} else {
							var closeclick_event = GEvent.addListener( map.getInfoWindow(), 'closeclick', function () { refreshKml(); GEvent.removeListener(closeclick_event); } );
						}
					}
				);
				GEvent.addListener( map, 'zoomend', function () { refreshKml(); } );
			}
		}
		if (q["sv"] != null && q["sv"] == 1 && w > 250 && h > 250) {
			self.document.getElementById("streetview-tool").innerHTML = '<a style="display: block; font-size: 1px; text-indent: -200px; overflow: hidden; cursor: pointer; border: none; margin: 0; padding: 0; position: absolute; top: 0; left: 56px; width: 15px; height: 15px; z-index: 10000; background-image: url(\'./gmap/gguy.gif\')" onclick="toggleStreetviewOverlay()" title="ストリートビュー">ストリートビュー</a>';
		}
		if (field_enabled || (q["s"] != null && q["s"] == 1)) {
			self.document.getElementById("geocoder-tool").innerHTML = '<a style="display: block; font-size: 1px; text-indent: -200px; overflow: hidden; cursor: pointer; border: none; margin: 0; padding: 0; position: absolute; top: 0; left: 0; width: 15px; height: 15px; z-index: 10000; background-image: url(\'./gmap/search.gif\')" onclick="toggleSearchBox()" title="住所検索">住所検索</a><form id="geocoder-search" action="#" onsubmit="geocoderSearch(); return false;" style="width: ' + w + 'px; display: none; position: absolute; top: 0; left: 0; margin-top: 100px; text-align: center;"><input type="search" placeholder="住所検索" results="5" autosave="geocodersearch" size="24" id="geocoderkey" value="" /><input type="button" onclick="geocoderSearch()" value="検索" /><img style="border: 0px none ; margin-left: 3px; width: 14px; height: 13px; -moz-user-select: none; cursor: pointer;" src="http://maps.gstatic.com/mapfiles/close.gif" onclick="toggleSearchBox()" title="住所検索欄を閉じる" /></form>';
			self.document.getElementById("geocoder-search").style.display = "none";
			if (field_enabled) {
				GEvent.addListener( map, 'moveend', function () { if (map.getInfoWindow().isHidden()) { centerMarker(); saveMap(); } } );
				GEvent.addListener( map, 'zoomend', function () { centerMarker(); saveMap(); } );
				GEvent.addListener( map, 'maptypechanged', function () { saveMap(); } );
			}
		}
		if (
			(
				m_field != null
				|| (q["m"] != null && q["m"] != "")
				|| kml_by_myself
				|| kml_overlay != null
			) && q["tr"] != null && q["tr"] != ""
		) {
			GEvent.addListener( map, 'infowindowopen', 
				function () {
					var self_links = document.getElementsByTagName('a');
					for (var i=0; i<self_links.length; i++) {
						if (
								typeof self_links[i].getAttribute("href") == "string"
								&& self_links[i].getAttribute("href").search(/^http/i) != -1
								&& self_links[i].getAttribute("href").search(/^http:\/\/\w+\.google/i) == -1
								&& (self_links[i].target == null || self_links[i].target == "")
						) {
							if (q["tr"] != null && q["tr"] == "p") {
								self_links[i].target = "_parent";
							} else if (q["tr"] != null && q["tr"] == "t") {
								self_links[i].target = "_top";
							} else if (q["tr"] != null && q["tr"] == "s") {
								self_links[i].target = "_self";
							} else if (q["tr"] != null && q["tr"] == "d") {
								if (self_links[i].getAttribute("href").search(/m=(DetailViewer|Inquiry)&/) != -1) {
									self_links[i].target = "detailViewer";
									self_links[i].onclick = function () { window.open('','detailViewer','width=640,height=480,scrollbars=yes,resizable=yes'); };
								} else {
									self_links[i].target = "_blank";
								}
							} else {
								self_links[i].target = "_blank";
							}
						}
					}
					var self_forms = document.getElementsByTagName('form');
					for (var i=0; i<self_forms.length; i++) {
						if (
								typeof self_forms[i].getAttribute("action") == "string"
								&& self_forms[i].getAttribute("action").search(/^http/i) != -1
								&& self_forms[i].getAttribute("action").search(/^http:\/\/\w+\.google/i) == -1
								&& (kml_by_myself || self_forms[i].target == null || self_forms[i].target == "")
						) {
							if (q["tr"] != null && q["tr"] == "p") {
								self_forms[i].target = "_parent";
							} else if (q["tr"] != null && q["tr"] == "t") {
								self_forms[i].target = "_top";
							} else if (q["tr"] != null && q["tr"] == "s") {
								self_forms[i].target = "_self";
							} else {
								self_forms[i].target = "_blank";
							}
						}
					}
				}
			);
		}
	} else if (q["key"] != null && q["key"] != "") {
		var w;
		var h;
		if (q["width"] != null && q["width"] != "" && !isNaN(q["width"]) && q["height"] != null && q["height"] != "" && !isNaN(q["height"])) {
			w = parseInt(q["width"]);
			h = parseInt(q["height"]);
		} else {
			w = ((self.innerWidth) ? self.innerWidth : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientWidth : self.document.body.clientWidth));
			h = ((self.innerHeight) ? self.innerHeight : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientHeight : self.document.body.clientHeight))
		}
		if (w > 512) w = 512;
		if (h > 512) h = 512;
		var gmap_img = document.createElement('img');
		gmap_img.style.width = w + "px";
		gmap_img.style.height = h + "px";
		gmap_img.style.border = "none";
		if (q["m"] != null && q["m"] != "") gmap_img.title = q["m"].replace(/<.*?>/,'');
		var center = ((q["c"] != null && q["c"] != "") ? q["c"] : '36.315125,137.504883');
		gmap_img.src	= 'http://maps.google.com/staticmap?key=' + q["key"]
						+ '&center=' + center
						+ '&zoom=' + ((q["z"] != null && q["z"] != "" && !isNaN(q["z"])) ? parseInt(q["z"]) : 4)
						+ '&size=' + w + 'x' + h
						+ ((q["m"] != null && q["m"] != "") ? '&markers=' + center : '');
		self.document.getElementById("map").appendChild(gmap_img);
		var gmap_msg = document.createElement('p');
		gmap_msg.style.position = 'absolute';
		gmap_msg.style.backgroundColor = '#FFF';
		gmap_msg.style.color = '#000';
		gmap_msg.style.top = 0;
		gmap_msg.style.left = 0;
		gmap_msg.innerHTML = 'Google マップを表示できませんでした。';
		self.document.getElementById("map").appendChild(gmap_msg);
	}
}

function makeCenterMarker(make_from_form) {
	if (center_marker != null) map.removeOverlay(center_marker);
	var center_icon = null;
	if (make_from_form) {
		var mk;
		var mka;
		var mkv;
		var mks;
		if (mk_field != null && (mkc_field == null || mkc_field.checked)) {
			if (mk_field.type.search(/select/) != -1) {
				mk = mk_field.options[mk_field.options.selectedIndex].value;
			} else if (mk_field.value != null && mk_field.value != "") {
				mk = mk_field.value;
			}
		}
		if (mka_field != null) {
			if (mka_field.type.search(/select/) != -1) {
				mka = mka_field.options[mka_field.options.selectedIndex].value;
			} else if (mk_field.value != null && mk_field.value != "") {
				mka = mka_field.value;
			}
		}
		if (mkv_field != null) {
			if (mkv_field.type.search(/select/) != -1) {
				mkv = mkv_field.options[mkv_field.options.selectedIndex].value;
			} else if (mk_field.value != null && mk_field.value != "") {
				mkv = mkv_field.value;
			}
		}
		if (mk != null && mk != "" && mk.search(/^https?:\/\/.*\.(gif|png|jpg|jpeg)$/i) != -1) {
			miniicon_disabled = true;
			center_icon = getCustomGIcon(mk,mka,mkv,null);
		} else if (q["mk"] != null && q["mk"] != "") {
			center_icon = getCustomGIcon(q["mk"],((mkc_field != null) ? mka : q["mka"]),((mkc_field != null) ? mkv : q["mkv"]),q["mks"]);
		} else {
			miniicon_disabled = false;
			center_icon = getGIcon();
		}
	} else if (q["mk"] != null && q["mk"] != "") {
		center_icon = getCustomGIcon(q["mk"],q["mka"],q["mkv"],q["mks"]);
	} else {
		center_icon = getGIcon();
	}
	map.enableDoubleClickZoom();
	center_marker = new GMarker(
		map_center,
		{
			icon	:	center_icon,
			title	:	'この地図の中心'
		}
	);
	GEvent.addListener(
		center_marker,
		"click",
		function () {
			map.disableDragging();
			map_center = map.getCenter();
			openCenterInfoWindow();
		}
	);
	GEvent.addListener(
		center_marker,
		"infowindowclose",
		function () {
			map.enableDragging();
			map.panTo(map_center);
		}
	);
	GEvent.addListener(
		map,
		"drag",
		function () {
			center_marker.setLatLng(map.getCenter());
		}
	);
	map.addOverlay(center_marker);
	centerMarker();
}

function getCustomGIcon(url,mka,mkv,surl) {
	var g_image = new Image();
	g_image.src = url;
	var w = g_image.width;
	var h = g_image.height;
	if (surl != null && surl != "") {
		var sg_image = new Image();
		sg_image.src = surl;
		var sw = sg_image.width;
		var sh = sg_image.height;
	}
	if (w != null && w > 0) {
		var g_icon = new GIcon();
		g_icon.image = url;
		g_icon.shadow = (surl != null && surl != "") ? surl : null;
		g_icon.iconSize = new GSize(w, h);
		g_icon.shadowSize = new GSize(((sw != null && sw > 0) ? sw : w*2), ((sh != null && sh > 0) ? sh : h));
		if (mka == null) mka = "center";
		if (mkv == null) mkv = "bottom";
		var a = (mka == "left") ? 0 : (mka == "right") ? w : parseInt(w/2);
		var v = (mkv == "top") ? 0 : (mkv == "center") ? parseInt(h/2) : (h+1);
		g_icon.iconAnchor = new GPoint(a, v);
		g_icon.infoWindowAnchor = new GPoint(parseInt(w/2), 0);
		miniicon_disabled = true;
		return g_icon;
	} else {
		return getGIcon();
	}
}

function getCustomGIconFromKml(icon_style) {
	var href = GXml.value(icon_style.getElementsByTagName("href")[0]);
	var width = parseInt(GXml.value(icon_style.getElementsByTagName("width")[0]));
	var height = parseInt(GXml.value(icon_style.getElementsByTagName("height")[0]));
	var anchorx = parseInt(GXml.value(icon_style.getElementsByTagName("anchorx")[0]));
	var anchory = parseInt(GXml.value(icon_style.getElementsByTagName("anchory")[0]));
	var shadow = GXml.value(icon_style.getElementsByTagName("shadow")[0]);
	var swidth = parseInt(GXml.value(icon_style.getElementsByTagName("swidth")[0]));
	var sheight = parseInt(GXml.value(icon_style.getElementsByTagName("sheight")[0]));
	if (href != "" && width > 0) {
		var g_icon = new GIcon();
		g_icon.image = href;
		g_icon.shadow = (shadow != "") ? shadow : null;
		g_icon.iconSize = new GSize(width, height);
		g_icon.shadowSize = new GSize(((swidth > 0) ? swidth : (width*2)), ((sheight > 0) ? sheight : height));
		g_icon.iconAnchor = new GPoint(anchorx, anchory);
		g_icon.infoWindowAnchor = new GPoint(parseInt(width/2), 0);
		return g_icon;
	} else {
		return null;
	}
}

function refreshKml() {
	var kml_refresh_url = kml_url;
	if ((q["r"] != null && q["r"] == 1) || kml_overlay != null || kml_objects != null) {
		kml_refresh_url += ((kml_refresh_url.search(/\?/) == -1) ? '?' : '&') + 'gmap_sw=' + map.getBounds().getSouthWest().lat() + ',' + map.getBounds().getSouthWest().lng() + '&gmap_ne=' + map.getBounds().getNorthEast().lat() + ',' + map.getBounds().getNorthEast().lng();
		if (kml_by_myself && kml_objects != null) {
			for (var i=0; i<kml_objects.length; i++) {
				map.removeOverlay(kml_objects[i]);
			}
			kml_objects = new Array();
		} else if (!kml_by_myself && kml_overlay != null) {
			map.removeOverlay(kml_overlay);
		}
	}
	if (!kml_by_myself) {
		kml_overlay = new GGeoXml(kml_refresh_url);
		map.addOverlay(kml_overlay);
		if (q["fit"] != null && q["fit"] == 1) {
			kml_overlay.gotoDefaultViewport(map);
			saveMap();
		}
	}
	if (kml_by_myself || (q["d"] != null && q["d"] >= 1)) {
		var request = GXmlHttp.create();
		request.open("GET", kml_refresh_url, true);
		request.onreadystatechange = function () {
			if (request.readyState == 4) {
				//var kml = request.responseXML;
				var kml = GXml.parse(request.responseText);
				if ((kml_by_myself && q["fit"] != null && q["fit"] == 1) || (q["d"] != null && q["d"] >= 1)) {
					var coordinates = kml.documentElement.getElementsByTagName("coordinates");
					var n;
					var e;
					var s;
					var w;
					for (var i=0; i<coordinates.length; i++) {
						var points = GXml.value(coordinates[i]).replace(/^\s+/,'').replace(/\s+$/,'').split(/\s+/);
						var len = points.length;
						var center_n = Math.floor(len/2);
						var center_point = null;
						var start_point = null;
						var prev_point = null;
						var distance = 0;
						for (var j=0; j<len; j++) {
							var lnglat = points[j].split(/,/);
							if (lnglat[0] != null && lnglat[1] != null) {
								var lng = parseFloat(lnglat[0]);
								var lat = parseFloat(lnglat[1]);
								if (kml_by_myself && q["fit"] != null && q["fit"] == 1) {
									if (n == null || lat > n) n = lat;
									if (s == null || lat < s) s = lat;
									if (e == null || lng > e) e = lng;
									if (w == null || lng < w) w = lng;
								}
								if (q["d"] != null && q["d"] >= 1) {
									var here = new GLatLng(lat,lng);
									if (start_point == null) start_point = here;
									if (j == center_n && len > 1) center_point = here;
									if (prev_point != null) distance += here.distanceFrom(prev_point);
									prev_point = here;
								}
							}
						}
						if (q["d"] != null && q["d"] >= 1 && center_point != null) {
							var distance_info = formattedDistance(distance);
							if (len == 2) {
								center_point = new GLatLng(parseFloat((center_point.lat() + start_point.lat()) / 2),parseFloat((center_point.lng() + start_point.lng()) / 2));
							} else if (q["d"] == 2) {
								var shortest = prev_point.distanceFrom(start_point);
								if (shortest != distance) distance_info += '<div style="font-size: 12px">(直線距離:' + formattedDistance(shortest) + ')</div>';
							}
							if (measure_icon == null) measure_icon = getMeasureIcon();
							map.addOverlay(makeMarker(center_point,measure_icon,distance_info));
						}
					}
				}
				if (kml_by_myself) {
					if (list != null) {
						if (list_box != null) list_box.style.display = "none";
						while (list.hasChildNodes()) {
							list.removeChild(list.lastChild)
						}
					}
					if (kml_objects == null) kml_objects = new Array();
					if (default_icon == null) {
						if (kml.documentElement.getElementsByTagName("Style")[0] != null) {
							var first_style = kml.documentElement.getElementsByTagName("Style")[0];
							if ((first_style.getAttribute("id") == "defaultIcon" || (nosync && first_style.getAttribute("id") == "temporaryIcon")) && first_style.getElementsByTagName("IconStyle")[0] != null) {
								default_icon = getCustomGIconFromKml(first_style.getElementsByTagName("IconStyle")[0]);
								miniicon_disabled = true;
							}
						}
						if (default_icon == null) default_icon = (q["mk"] != null && q["mk"] != "") ? getCustomGIcon(q["mk"],q["mka"],q["mkv"],q["mks"]) : getGIcon();
					}
					var i = 0;
					var network_links = kml.documentElement.getElementsByTagName("NetworkLink");
					for (i=0; i<network_links.length; i++) {
						var kml_link = GXml.value(network_links[i].getElementsByTagName("href")[0]);
						if (kml_link != null) {
							var kml_layer = new GGeoXml(kml_link);
							map.addOverlay(kml_layer);
							kml_objects.push(kml_layer);
						}
					}
					var placemarks = kml.documentElement.getElementsByTagName("Placemark");
					var center_placemark = null;
					for (i=0; i<placemarks.length; i++) {
						var placemarker;
						var placemark_icon = null;
						var placemark_name = GXml.value(placemarks[i].getElementsByTagName("name")[0]);
						var placemark_description = GXml.value(placemarks[i].getElementsByTagName("description")[0]);
						var placemark_link = (placemarks[i].getElementsByTagName("atom:link")[0] != null) ? placemarks[i].getElementsByTagName("atom:link")[0].getAttribute("href") : (placemarks[i].getElementsByTagName("link")[0] != null) ? placemarks[i].getElementsByTagName("link")[0].getAttribute("href") : null;
						var placemark_id = (placemarks[i].getElementsByTagName("atom:id")[0] != null) ? GXml.value(placemarks[i].getElementsByTagName("atom:id")[0]) : (placemarks[i].getElementsByTagName("id")[0] != null) ? GXml.value(placemarks[i].getElementsByTagName("id")[0]) : null;
						var placemark_lnglat = GXml.value(placemarks[i].getElementsByTagName("coordinates")[0]).split(/,/);
						var placemark_point = (placemark_lnglat.length <= 1) ? map.getCenter() : new GLatLng(parseFloat(placemark_lnglat[1]),parseFloat(placemark_lnglat[0]));
						if (placemarks[i].getElementsByTagName("IconStyle")[0] != null) placemark_icon = getCustomGIconFromKml(placemarks[i].getElementsByTagName("Style")[0]);
						placemarker = makeMarker(placemark_point,((placemark_icon == null) ? default_icon : placemark_icon),placemark_name,placemark_description);
						map.addOverlay(placemarker);
						if (!nosync && placemark_link != null && placemark_link != "") {
							setMouseOverImage(
								placemark_link,
								placemarker,
								placemark_name,
								placemark_description,
								((placemark_icon == null) ? miniicon_disabled : true)
							);
						}
						kml_objects.push(placemarker);
						if (q["opn"] != null && (q["opn"] == placemark_id || q["opn"] == placemark_link)) {
							center_placemark = {
								"link"				:	placemark_link,
								"marker"			:	placemarker,
								"name"				:	placemark_name,
								"description"		:	placemark_description,
								"is_custom_icon"	:	((placemark_icon == null) ? miniicon_disabled : true)
							};
							q["opn"] = null;
						}
						if (list != null && placemark_link != null) {
							var list_a = _caller.document.createElement("a");
							list_a.href = placemark_link;
							if (q["tr"] != null && q["tr"] == "p") {
								list_a.target = "_self";
							} else if (q["tr"] != null && q["tr"] == "t") {
								list_a.target = "_top";
							} else if (q["tr"] != null && q["tr"] == "s") {
								list_a.target = "_self";
							} else if (q["tr"] != null && q["tr"] == "d") {
								list_a.target = "detailViewer";
								list_a.onclick = function () { window.open('','detailViewer','width=640,height=480,scrollbars=yes,resizable=yes'); };
							} else {
								list_a.target = "_blank";
							}
							setMouseOverImage(
								placemark_link,
								placemarker,
								placemark_name,
								placemark_description,
								((placemark_icon == null) ? miniicon_disabled : true),
								[list_a]
							);
							var list_img = _caller.document.createElement('img');
							list_img.src = placemarker.getIcon().image;
							list_a.appendChild(list_img);
							list_a.appendChild(_caller.document.createTextNode(placemark_name));
							var list_li = _caller.document.createElement('li');
							list_li.appendChild(list_a);
							list.appendChild(list_li);
						}
					}
					var screen_overlays = kml.documentElement.getElementsByTagName("ScreenOverlay");
					for (var i=0; i<screen_overlays.length; i++) {
						var screen_overlay;
						var screen_overlay_name = GXml.value(screen_overlays[i].getElementsByTagName("name")[0]);
						var screen_overlay_description = GXml.value(screen_overlays[i].getElementsByTagName("description")[0]);
						if (screen_overlays[i].getElementsByTagName("overlayXY")[0] != null) {
							var thisOverlayXY = screen_overlays[i].getElementsByTagName("overlayXY")[0];
							var overlayXY = new GScreenPoint(
								parseFloat(thisOverlayXY.getAttribute("x")),
								parseFloat(thisOverlayXY.getAttribute("y")),
								thisOverlayXY.getAttribute("xunits"),
								thisOverlayXY.getAttribute("yunits")
							);
						}
						if (screen_overlays[i].getElementsByTagName("screenXY")[0] != null) {
							var thisScreenXY = screen_overlays[i].getElementsByTagName("screenXY")[0];
							var screenXY = new GScreenPoint(
								parseFloat(thisScreenXY.getAttribute("x")),
								parseFloat(thisScreenXY.getAttribute("y")),
								thisScreenXY.getAttribute("xunits"),
								thisScreenXY.getAttribute("yunits")
							);
						}
						if (screen_overlays[i].getElementsByTagName("size")[0] != null) {
							var thisSize = screen_overlays[i].getElementsByTagName("size")[0];
							var size = new GScreenSize(
								parseFloat(thisSize.getAttribute("width")),
								parseFloat(thisSize.getAttribute("height"))
							);
						}
						if (screen_overlays[i].getElementsByTagName("Icon")[0] != null) {
							var screen_overlay_icon = GXml.value(screen_overlays[i].getElementsByTagName("Icon")[0].getElementsByTagName("href")[0]);
						}
						screen_overlay = new GScreenOverlay(
							screen_overlay_icon,
							screenXY,
							overlayXY,
							size
						);
						map.addOverlay(screen_overlay);
						kml_objects.push(screen_overlay);
					}
					if (q["fit"] != null && q["fit"] == 1 && n != null && s != null && e != null && w != null) {
						var bound = new GLatLngBounds(new GLatLng(s,w),new GLatLng(n,e));
						map.setCenter(bound.getCenter(), map.getBoundsZoomLevel(bound));
						saveMap();
					}
					if (center_placemark != null) {
						if (!center_placemark.is_custom_icon) toggleGIcon(center_placemark.marker,true);
						if ((center_placemark.description != null && center_placemark.description != "") || (center_placemark.title != null && center_placemark.title != "")) {
							center_placemark.marker.openInfoWindowHtml(
								((center_placemark.description != null && center_placemark.description != "") ? center_placemark.description : center_placemark.title),
								{maxWidth : mw}
							);
						}
					}
					if (list != null && list.hasChildNodes() && list_box != null) list_box.style.display = "";
				}
			}
		}
		request.send(null);
	}
}

function setMouseOverImage(placemark_link,placemarker,marker_title,marker_description,is_custom_icon,caller_links) {
	if (!caller_links) caller_links = _caller.document.getElementsByTagName('a');
	for (var i=0; i<caller_links.length; i++) {
		if (caller_links[i].href == placemark_link) {
			GEvent.addDomListener(
				caller_links[i],
				'mouseover',
				function () {
					if (!is_custom_icon) toggleGIcon(placemarker,true);
					if ((marker_description != null && marker_description != "") || (marker_title != null && marker_title != "")) {
						placemarker.openInfoWindowHtml(
							((marker_description != null && marker_description != "") ? marker_description : marker_title),
							{maxWidth : mw}
						);
					}
				}
			);
			if (!is_custom_icon) {
				GEvent.addDomListener(
					caller_links[i],
					'mouseout',
					function () {
						toggleGIcon(placemarker,false);
					}
				);
			}
		}
	}
}

function openPanoramaBubble() {
	var w = ((self.innerWidth) ? self.innerWidth : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientWidth : self.document.body.clientWidth));
	var h = ((self.innerHeight) ? self.innerHeight : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientHeight : self.document.body.clientHeight));
	var content_node = document.createElement('div');
	w -= 42;
	h -= 141;
	content_node.style.textAlign = 'center';
	content_node.style.width = ((w > 656) ? 656 : w) + 'px';
	content_node.style.height = ((h > 615) ? 615 : h) + 'px';
	content_node.innerHTML = '読み込み中...';

	var small_node = document.createElement('div');
	small_node.style.width = '220px';
	small_node.style.height = '200px';
	streetview_man.openInfoWindow(small_node, { maxContent: content_node, maxTitle: "最大表示" });

	var panorama = new GStreetviewPanorama(small_node);
	panorama.setLocationAndPOV(streetview_man.getLatLng(),streetview_man_pov);
	GEvent.addListener(
		panorama,
		"initialized",
		function(new_location) {
			if (new_location.latlng != null && (new_location.latlng.lat() != streetview_man_location.lat() || new_location.latlng.lng() != streetview_man_location.lng())) {
				var lat_gap = new_location.latlng.lat() - streetview_man_last.lat();
				var lng_gap = new_location.latlng.lng() - streetview_man_last.lng();
				var map_center = map.getCenter();
				streetview_man_location = new_location.latlng;
				streetview_man.setLatLng(streetview_man_location);
				map.getInfoWindow().restore(); // 最大化して移動すると元に戻ります。
				map.getInfoWindow().reset(streetview_man_location,null,null,map.getInfoWindow().getPixelOffset(),null);
				map.panTo(new GLatLng((map_center.lat() + lat_gap),(map_center.lng() + lng_gap)));
				streetview_man_last = streetview_man_location;
				streetview_man_pov = panorama.getPOV();
			}
		}
	);
	GEvent.addListener(
		panorama,
		"yawchanged",
		function(newYaw) {
			var GUY_NUM_ICONS = 16;
			var GUY_ANGULAR_RES = 360/GUY_NUM_ICONS;
			if (newYaw < 0) newYaw += 360;
			streetview_man.setImage("http://maps.gstatic.com/mapfiles/cb/man_arrow-" + (Math.round(newYaw/GUY_ANGULAR_RES) % GUY_NUM_ICONS) + ".png");
			streetview_man_pov = panorama.getPOV();
		}
	); 	
	GEvent.addListener(
		panorama,
		"pitchchanged",
		function() {
			streetview_man_pov = panorama.getPOV();
		}
	); 	
	GEvent.addListener(
		panorama,
		"zoomchanged",
		function() {
			streetview_man_pov = panorama.getPOV();
		}
	); 	
	GEvent.addListener(
		map.getInfoWindow(),
		"maximizeend",
		function() {
			content_node.innerHTML = '';
			panorama.setContainer(content_node);
			window.setTimeout(function(){ panorama.checkResize() }, 5);
		}
	);
	GEvent.addListener(
		map.getInfoWindow(),
		"restoreend",
		function() {
			panorama.setContainer(small_node);
			window.setTimeout(function(){ panorama.checkResize() }, 5);
		}
	);
}

function toggleStreetviewOverlay() {
	if (streetview_overlay == null) {
		if (streetview_client == null) streetview_client = new GStreetviewClient();
		streetview_man_location = null;
		streetview_man = new GMarker(map.getCenter(), { icon: getGuyIcon(), draggable: true });
		map.addOverlay(streetview_man);
		GEvent.addListener(streetview_man, "dragstart", function() { streetview_man.closeInfoWindow(); });
		GEvent.addListener(streetview_man, "dragend", getNearestStreetview);
		GEvent.addListener(streetview_man, "click", getNearestStreetview);
		GEvent.addListener(streetview_man, "drag", function() {
			if (!streetview_man_last || !streetview_man_last.lng()) {
				streetview_man.setImage("http://maps.gstatic.com/mapfiles/cb/man_fly_right.png");
			} else {
				var lng_gap = streetview_man.getLatLng().lng() - streetview_man_last.lng();
				var lat_gap = streetview_man.getLatLng().lat() - streetview_man_last.lat();
				if (lng_gap > 0) {
					streetview_man.setImage("http://maps.gstatic.com/mapfiles/cb/man_fly_right.png");
				} else if (streetview_man_last.lng() > streetview_man.getLatLng().lng()) {
					streetview_man.setImage("http://maps.gstatic.com/mapfiles/cb/man_fly_left.png");
				}
				if (Math.abs(lat_gap) > Math.abs(lng_gap)) {
					streetview_man_pov.yaw = (lat_gap > 0) ? 0 : 180;
				} else {
					streetview_man_pov.yaw = (lng_gap > 0) ? 90 : 270;
				}
			}
			streetview_man_last = streetview_man.getLatLng();
		});
		getNearestStreetview();
		streetview_overlay = new GStreetviewOverlay();
		map.addOverlay(streetview_overlay);
	} else {
		map.removeOverlay(streetview_overlay);
		streetview_overlay = null;
		map.removeOverlay(streetview_man);
		streetview_man = null;
	}
}

function getNearestStreetview() {
	streetview_man.closeInfoWindow();
	streetview_client.getNearestPanoramaLatLng(streetview_man.getLatLng(), onResponse);
}

function onResponse(latlng) {
	if (!latlng) {
		if (streetview_man_location != null) {
			streetview_man.setLatLng(streetview_man_location);
			streetview_man_last = streetview_man_location;
		}
	} else {
		streetview_man.setLatLng(latlng);
		streetview_man_location = latlng;
		streetview_man_last = latlng;
		openPanoramaBubble();
	}
}

function formattedDistance (meter) {
	return (meter > 1000) ? (parseInt(meter) / 1000) + 'km' : parseInt(meter) + 'm';
}

function openCenterInfoWindow() {
	if (m_field.value != "") center_marker.openInfoWindowHtml('<div style="font-size: 14px">' + m_field.value + '</div>',{maxWidth : mw});
}

function makeMarker(point,icon,marker_title,marker_description){
	var marker = new GMarker(point,{
		icon	:	icon,
		title	:	marker_title.replace(/<.*?>/g,'')
	});
	GEvent.addListener(
		marker,
		"click",
		function() {
			marker.openInfoWindowHtml(
				((marker_description != null && marker_description != "") ? marker_description : marker_title),
				{maxWidth : mw}
			);
		}
	);
	return marker;
}

function geocoderSearch() {
	var keyword = self.document.getElementById('geocoderkey').value;
	if (keyword != null && keyword != "") {
		if (geocoder_client == null) geocoder_client = new GClientGeocoder();
		geocoder_client.getLatLng(
			keyword,
			function (latlng) {
				if (!latlng) {
					alert("条件に該当する地図は見つかりませんでした");
				} else {
					var z = map.getZoom();
					map.setCenter(latlng, ((z < 10) ? 15 : z));
					saveMap();
					centerMarker();
				}
			}
		);
	}
}

function toggleSearchBox() {
	var search_box = self.document.getElementById("geocoder-search");
	if (search_box.style.display == null || search_box.style.display == 'none') {
		var addr = '';
		if (addr_field_ids != null && addr_field_ids.constructor == Array && addr_field_ids.length > 0) {
			for (var i in addr_field_ids) {
				if (_caller.document.getElementById(addr_field_ids[i])) {
					var addr_field = _caller.document.getElementById(addr_field_ids[i])
					if (addr_field.type != null && addr_field.type == 'select') {
						addr += addr_field.options[addr_field.options.selectedIndex].value;
					} else {
						addr += addr_field.value;
					}
				}
			}
			if (addr != "") self.document.getElementById("geocoderkey").value = addr;
		}
		search_box.style.display = 'block';
		self.document.getElementById("geocoderkey").focus();
		if (addr == "") self.document.getElementById("geocoderkey").select();
	} else {
		search_box.style.display = 'none';
	}
}

function saveMap() {
	if (z_field != null) z_field.value = map.getZoom();
	if (t_field != null) t_field.value = map.getCurrentMapType().getUrlArg();
	if (c_field != null) c_field.value = map.getCenter().lat() + ',' + map.getCenter().lng();
	if (fit_field != null) fit_field.checked = false;
}

function centerMarker() {
	if (center_marker != null) center_marker.setLatLng(map.getCenter());
}

function fitWindow() {
	var w = ((self.innerWidth) ? self.innerWidth : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientWidth : self.document.body.clientWidth));
	var h = ((self.innerHeight) ? self.innerHeight : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientHeight : self.document.body.clientHeight));
	self.document.getElementById("map").style.width = w + "px";
	self.document.getElementById("map").style.height = h + "px";
	if (map != null) map.checkResize();
}

function urlDecode(query) {
	var hash = new Array();
	if (query == null || query == "" || query.search("=") == -1) return hash;
	query = query.replace(/^\?/,'').split('&');
	for (var i=0; i<query.length; i++) {
		var p = query[i].split('=');
		if (window.decodeURIComponent) {
			p[0] = decodeURIComponent(p[0].replace(/\+/g,' '));
			p[1] = decodeURIComponent(p[1].replace(/\+/g,' '));
		} else {
			p[0] = unescape(p[0].replace(/\+/g,' '));
			p[1] = unescape(p[1].replace(/\+/g,' '));
		}
		if (hash[p[0]] == null) {
			hash[p[0]] = p[1];
		} else {
			if (hash[p[0]].constructor == Array) {
				hash[p[0]].push(p[1]);
			} else {
				hash[p[0]] = new Array(hash[p[0]],p[1]);
			}
		}
	}
	return hash;
}

function urlEncode (hash) {
	var query = '';
	for (var key in hash) {
		if (hash[key] != null) {
			var encoded_key = ((window.encodeURIComponent) ? encodeURIComponent(key) : escape(key)).replace(/ /g,'+')
			if (hash[key].constructor == Array) {
				for (var i=0; i<hash[key].length; i++) {
					if ((typeof hash[key][i] == "string" || typeof hash[key][i] == "number") && hash[key][i] != null) query += ((query != "") ? "&" : "") + encoded_key + "=" + ((window.encodeURIComponent) ? encodeURIComponent(hash[key][i]) : escape(hash[key][i])).replace(/ /g,'+');
				}
			} else if (typeof hash[key] == "string" || typeof hash[key] == "number") {
				query += ((query != "") ? "&" : "") + encoded_key + "=" + ((window.encodeURIComponent) ? encodeURIComponent(hash[key]) : escape(hash[key])).replace(/ /g,'+');
			}
		}		
	}
	return query;
}

function getHashFromForm (this_form) {
	var hash = new Array();
	for (var i=0; i < this_form.elements.length; i++) {
		if (typeof this_form.elements[i].name != "undefined" && this_form.elements[i].name != "" && (typeof this_form.elements[i].value == "string" || typeof this_form.elements[i].value == "number")) {
			var name = this_form.elements[i].name;
			if (this_form.elements[i].type == null || typeof this_form.elements[i].type == "undefined") {
				hash[name] = _setHashValue(hash[name],this_form.elements[i].value);
			} else if (this_form.elements[i].type == 'text' || this_form.elements[i].type == 'password' || this_form.elements[i].type == 'textarea' || this_form.elements[i].type == 'hidden' || this_form.elements[i].type == 'submit' || this_form.elements[i].type == 'image' || this_form.elements[i].type == 'search') {
				if (this_form.elements[i].type == 'hidden' && this_form.elements[i].getAttribute('alt') && typeof this_form.elements[i].getAttribute('alt') == 'string') {
					hash[name] = _setHashValue(hash[name],this_form.elements[i].getAttribute('alt'));
				} else {
					hash[name] = _setHashValue(hash[name],this_form.elements[i].value);
				}
			} else if (this_form.elements[i].type == 'checkbox' || this_form.elements[i].type == 'radio') {
				if (this_form.elements[i].checked) hash[name] = _setHashValue(hash[name],this_form.elements[i].value);
			} else if (this_form.elements[i].type.search(/select/) != -1) {
				for (var o=0; o < this_form.elements[i].options.length; o++) {
					if (this_form.elements[i].options[o].selected) hash[name] = _setHashValue(hash[name],this_form.elements[i].options[o].value);
				}
			}
		}
	}
	return hash;
}

function _setHashValue (p,v) {
	if (p != null) {
		if (p.constructor == Array) {
			p.push(v);
		} else {
			p = new Array(p,v);
		}
	} else {
		p = v;
	}
	return p;
}